﻿using FlowerShopContracts.BindingModels;
using FlowerShopContracts.SearchModels;
using FlowerShopContracts.StoragesContracts;
using FlowerShopContracts.ViewModels;
using FlowerShopDatabaseImplement.Models;

namespace FlowerShopDatabaseImplement.Implements
{
    public class MessageInfoStorage : IMessageInfoStorage
    {
        public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
        {
            using var context = new FlowerShopDatabase();

            if (model.MessageId != null)
            {
                return context.Messages.FirstOrDefault(x => x.MessageId == model.MessageId)?.GetViewModel;
            }

            return null;
        }

        public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
        {
            using var context = new FlowerShopDatabase();
            return context.Messages
                .Where(x => x.ClientId == model.ClientId)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<MessageInfoViewModel> GetFullList()
        {
            using var context = new FlowerShopDatabase();
            return context.Messages
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
        {
            using var context = new FlowerShopDatabase();

            var newMessage = MessageInfo.Create(model);
            if (newMessage == null || context.Messages.Any(x => x.MessageId.Equals(model.MessageId)))
            {
                return null;
            }

            context.Messages.Add(newMessage);
            context.SaveChanges();

            return newMessage.GetViewModel;
        }
    }
}
